unit http_main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls,
  IdSSLOpenSSL, IdHTTP, IdComponent, IdSSLOpenSSLHeaders, IdCTypes;

type
  { TFHttp }

  TFHttp = class(TForm)
    BAdd: TButton;
    BDel: TButton;
    BGet: TButton;
    CbUrl: TComboBox;
    Http: TIdHTTP;
    IOHandlerSocketOpenSSL: TIdSSLIOHandlerSocketOpenSSL;
    LbUrl: TLabel;
    Memo1: TMemo;
    procedure BAddClick(Sender: TObject);
    procedure BDelClick(Sender: TObject);
    procedure BGetClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure IOHandlerSocketOpenSSLStatus(ASender: TObject;
      const AStatus: TIdStatus; const AStatusText: string);
    procedure IOHandlerSocketOpenSSLStatusInfo(const AMsg: String);
    procedure IOHandlerSocketOpenSSLStatusInfoEx(ASender: TObject;
      const AsslSocket: PSSL; const AWhere, Aret: TIdC_INT; const AType,
      AMsg: String);
    function IOHandlerSocketOpenSSLVerifyPeer(Certificate: TIdX509;
      AOk: Boolean; ADepth, AError: Integer): Boolean;
  private
    urlfn: String;
    procedure UrlListLoad;
    function UrlListFilename: String;
  public
    procedure InfoClear;
    procedure Info(const AText: String); overload;
    procedure Info(const AFormatText: String; const Args: array of const); overload;
  end;

var
  FHttp: TFHttp;

implementation

uses
  tb_indy_cert_helper;
{$R *.lfm}

{ TFHttp }

procedure TFHttp.BAddClick(Sender: TObject);
begin
  if CbUrl.Items.IndexOf(CbUrl.Text) < 0 then begin
    CbUrl.Items.Add(CbUrl.Text);
    CbUrl.Items.SaveToFile(UrlListFilename);
    BDel.Enabled:= True;
  end;
end;

procedure TFHttp.BDelClick(Sender: TObject);
var
  p: Integer;
begin
  p:= CbUrl.Items.IndexOf(CbUrl.Text);
  if p >= 0 then begin
    CbUrl.Items.Delete(p);
    CbUrl.Items.SaveToFile(UrlListFilename);
  end;
  BDel.Enabled:= CbUrl.Items.Count > 0;
end;

procedure TFHttp.BGetClick(Sender: TObject);
var
  s: TStringStream;
begin
  try
    InfoClear;
    s:= TStringStream.Create('');
    Http.HandleRedirects:= True;
    Http.RedirectMaximum:= 15;
    Http.Request.UserAgent:= 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/20100101 Firefox/12.0';
// Http.Request.RawHeaders.;
    Http.Get(CbUrl.Text, s);
    //Info('=======================DataString================================');
    //Info(s.DataString);
    //Info('');
    Info('=======================AcceptPatch=================================');
    Info(Http.Response.AcceptPatch);
    Info('');
    Info('=======================ResponseText================================');
    Info(Http.Response.ResponseText);
    Info('');
    Info('======================RawHeadersText===============================');
    Info(Http.Response.RawHeaders.Text);
  finally
    FreeAndNil(s);
  end;
end;

procedure TFHttp.UrlListLoad;
begin
  if FileExists(UrlListFilename) then begin
    CbUrl.Items.LoadFromFile(UrlListFilename);
  end;
end;

function TFHttp.UrlListFilename: String;
begin
  if urlfn = '' then begin
    urlfn:= GetUserDir+'http_url_list.txt';
  end;
  Result:= urlfn;
end;

procedure TFHttp.FormCreate(Sender: TObject);
begin
  UrlListLoad;
end;

procedure TFHttp.FormDestroy(Sender: TObject);
begin
  CbUrl.Items.SaveToFile(UrlListFilename);
end;

procedure TFHttp.IOHandlerSocketOpenSSLStatus(ASender: TObject;
  const AStatus: TIdStatus; const AStatusText: string);
begin
  if IOHandlerSocketOpenSSL.Tag = 0 then begin
    //  Root Zertifikate hinzufügen
    //  StoreCertificatsPathE steckt in TIdSSLContextHelper
    IOHandlerSocketOpenSSL.SSLContext.StoreCertificatsPathE(
        GetCurrentDir + DirectorySeparator + 'certs');
    //  Nur einmal im Status Event.
    //  "Tag" wird in "IOHandlerSocketOpenSSLVerifyPeer" wieder auf 0 gesetzt
    IOHandlerSocketOpenSSL.Tag:= 1;
  end;
  Info('SSL Status=%s', [AStatusText]);
end;

procedure TFHttp.IOHandlerSocketOpenSSLStatusInfo(const AMsg: String);
begin
  Info('SSL StatusInfo=%s', [AMsg]);
end;

procedure TFHttp.IOHandlerSocketOpenSSLStatusInfoEx(ASender: TObject;
  const AsslSocket: PSSL; const AWhere, Aret: TIdC_INT; const AType,
  AMsg: String);
begin
  Info('SSL StatusInfoEx=%s', [AMsg]);
end;

function TFHttp.IOHandlerSocketOpenSSLVerifyPeer(Certificate: TIdX509;
  AOk: Boolean; ADepth, AError: Integer): Boolean;
begin
  Info('IOHandlerSocketOpenSSLVerifyPeer:: AOk=%d, ADepth=%d, AError=%d',
    [Integer(AOk), ADepth, AError]);
  Info(Certificate.DisplayInfo.Text);
  Info('');
  Result:= AOk;
  IOHandlerSocketOpenSSL.Tag:= 0;
end;

procedure TFHttp.InfoClear;
begin
  Memo1.Clear;
end;

procedure TFHttp.Info(const AText: String);
begin
  Memo1.Lines.Append(AText);
end;

procedure TFHttp.Info(const AFormatText: String; const Args: array of const);
begin
  Memo1.Lines.Append(Format(AFormatText, Args));
end;

end.

