unit tb_indy_cert_helper;

{
  File: tb_indy_cert_helper.pas
  CreateDate: 21.05.2024
  ModifyDate: 21.05.2024
  Version: 1.0.0.0
  Author: Thomas Bulla

  See the file COPYING.TXT, included in this distribution,
  for details about the copyright.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
}


{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, IdSSLOpenSSL, IdSSLOpenSSLHeaders;

type
  EIndyCertHelper = class(Exception);

  { TIdSSLContextHelper }

  TIdSSLContextHelper = class helper for TIdSSLContext
  private
    type
      TIdSSLContextWrapper = class(TIdSSLContext);
    function IdSSLContextGetE: TIdSSLContextWrapper;
    function CertStoreGetE: PX509_STORE;
    procedure X509_STORE_load_locationsE(const aFileName, aPathName: String);
  public
    procedure StoreCertificatE(const CertFileName: String);
    procedure StoreCertificatsPathE(const PathName: String);
  end;


implementation

uses
  IdCTypes, IdGlobal;

{ TIdSSLContextHelper }

function TIdSSLContextHelper.IdSSLContextGetE: TIdSSLContextWrapper;
begin
  if Assigned(self) then begin
    Result:= TIdSSLContextWrapper(Self);
  end else begin
    raise EIndyCertHelper.Create('SslContext not assigned');
  end;
end;

function TIdSSLContextHelper.CertStoreGetE: PX509_STORE;
begin
  Result:= IdSSLContextGetE.FContext^.cert_store;
  if not Assigned(Result) then begin
    raise EIndyCertHelper.Create('Cert_Store not assigned');
  end;
end;

procedure TIdSSLContextHelper.X509_STORE_load_locationsE(
  const aFileName, aPathName: String);
var
  res: TIdC_INT;
begin
  res:= X509_STORE_load_locations(CertStoreGetE,
    PAnsiChar(Pointer(aFileName)),
    PAnsiChar(Pointer(aPathName)));
  if res <> 1 then begin
    raise EIndyCertHelper.CreateFmt('StoreCertificat fails, error=%d', [res]);
  end;
end;

procedure TIdSSLContextHelper.StoreCertificatE(
  const CertFileName: String);
begin
  X509_STORE_load_locationsE(CertFileName, '');
end;

procedure TIdSSLContextHelper.StoreCertificatsPathE(
  const PathName: String);
var file_info : TSearchRec;
    path: String;
begin
//  X509_STORE_load_locationsE('', PathName); //Sorry, doesn't work!
  path:= IncludeTrailingPathDelimiter(PathName);
  if FindFirst(path + '*',
       faNormal, file_info) = 0 then try
    repeat
      StoreCertificatE(path + file_info.Name);
    until FindNext(file_info) <> 0;
  finally
    FindClose(file_info);
  end;
end;

end.

